@@ -8,9 +8,9 @@ Please find a description of this package at:
 
 Thank you for using bitarray.
 """
-__version__ = '0.2.0'
+__version__ = '0.2.1'
 
-from _bitarray import _bitarray
+from _bitarray import _bitarray, bits2bytes
 
 
 def _getIndicesEx(r, length):
@@ -42,8 +42,7 @@ def _getIndicesEx(r, length):
         if stop < 0: stop = -1
         if stop > length: stop = length
 
-    if (step < 0 and stop >= length) or \
-       (step > 0 and start >= stop):
+    if (step < 0 and stop >= length) or (step > 0 and start >= stop):
         slicelength = 0
     elif step < 0:
         slicelength = (stop-start+1) / step + 1
@@ -57,47 +56,40 @@ def _getIndicesEx(r, length):
 
 
 class bitarray(_bitarray):
-    """bitarray([object][endian=string]) -> bitarray
+    """bitarray([initial][endian=string])
 
-    Return a new bitarray object whose items are bits initialized from
-    the optional object, and endianness.
-    If no object is provided, the bitarray is initalized to length zero.
-    The object may be of the following types:
-    
-    * int, long:  Create bitarray of length given by the integer.
-                  The initial values in the array are random,
-                  because only the memory allocated.
+Return a new bitarray object whose items are bits initialized from
+the optional initial, and endianness.
+If no object is provided, the bitarray is initalized to have length zero.
+The initial object may be of the following types:
 
-    * string:     Create bitarray from a string with '0's and '1's.
-                  Example: bitarray('01101001111')
+int, long
+    Create bitarray of length given by the integer.  The initial values
+    in the array are random, because only the memory allocated.
 
-    * sequence,   Create bitarray from a sequence, each element in the
-      list,       sequence is converted to a bit using the bool function.
-      tuple:      Example: bitarray([2, 0, 'a', {}])  ->  bitarray('1010')
+string
+    Create bitarray from a string with '0's and '1's,
+    e.g. bitarray('01101001111').
 
-    * bitarray:   Create bitarray from another bitarray.  This is done by
-                  copying the memory holding the bitarray data, and is hence
-                  very fast.
-    
-    The optional keyword arguments 'endian' specifies the bit endianness
-    of the created bitarray object.
-    Allowed values are 'big' and 'little' (default 'big')
-    
-    Note that setting the bit endianness only has an effect when accessing
-    the machine representation of the bitarray, i.e. when using the
-    methods: tofile, fromfile, tostring, fromstring
-    """
-    
-    def __len__(self):
-        """
-        Return the number of elements in the bitarray, not that for
-        very large bitarrays this may fail but .length() will still work.
-        """
-        return self.length()
+list, tuple, iterable
+    Create bitarray from a sequence, each element in the sequence is
+    converted to a bit using the bool function, 
+    e.g. bitarray([2, 0, 'a', {}])  ->  bitarray('1010')
 
+bitarray
+    Create bitarray from another bitarray.  This is done by copying the
+    memory holding the bitarray data, and is hence very fast.
+
+The optional keyword arguments 'endian' specifies the bit endianness of the
+created bitarray object.  Allowed values are 'big' and 'little' (default 'big')
+
+Note that setting the bit endianness only has an effect when accessing the
+machine representation of the bitarray, i.e. when using the methods: tofile,
+fromfile, tostring, fromstring
+"""
 
     def __hash__(self):
-        return hash(tuple(self))
+        return hash(self.to01())
 
     
     def __add__(self, other):
@@ -194,28 +186,31 @@ class bitarray(_bitarray):
 Read n bytes from the file object f and append them to the bitarray
 interpreted as machine values.  When n is omitted, as many bytes are
 read until EOF is reached."""
-        if self.length() % 8 == 0:
-            _bitarray.fromfile(self, f, n)
-        else:
-            ls1 = self.length()
-            self.fill()
-            ls2 = self.length()
-            _bitarray.fromfile(self, f, n)
-            del self[ls1:ls2]
+        ls1 = self.length()
+        self.fill()
+        ls2 = self.length()
+        _bitarray.fromfile(self, f, n)
+        del self[ls1:ls2]
 
 
-    def fromstring(self, s):
-        """fromstring(string)
+    def fromlist(self, lst):
+        """fromlist(list)
 
-Append from a string, interpreting the string as machine values."""
-        if self.length() % 8 == 0:
-            _bitarray.fromstring(self, s)
-        else:
-            ls1 = self.length()
-            self.fill()
-            ls2 = self.length()
-            _bitarray.fromstring(self, s)
-            del self[ls1:ls2]
+Append bits to bitarray from list.
+This method is **deprecated**, use the 'extend' method instead."""
+        if not isinstance(lst, list):
+            raise TypeError("list expected")
+        self.extend(lst)
+
+
+    def from01(self, s):
+        """from01(string)
+
+Appends items from the string, containing '0's and '1's, to the bitarray.
+This method is **deprecated**, use 'extend' method instead."""
+        if not isinstance(s, str):
+            raise TypeError("string expected")
+        self.extend(s)
 
 
     def insert(self, i, x):
@@ -223,7 +218,7 @@ Append from a string, interpreting the string as machine values."""
 
 Insert a new item x into the bitarray before position i."""
         if not isinstance(i, (int, long)):
-            raise TypeError("first argument must be int or long")
+            raise TypeError("first argument must be integer")
         self[i:i] = bitarray([x])
 
 
@@ -253,27 +248,10 @@ Return the i-th element and delete it from the bitarray. i defaults to -1."""
         raise IndexError("pop index out of range")
 
 
-    def reverse(self):
-        """reverse()
 
-reverse the order of bits in the bitarray."""
-        # TODO:
-        #   This is very slow, it needs to be implemented in C.
-        #   Also, we can be more memory efficient:
-        #      1. Copy first half of array into a new array
-        #      2. Reverse the second half onto the first.
-        #      3. Revert the stored away first half.
-        for i, b in enumerate(self[::-1]):
-            self[i] = b
+def test(verbosity=1):
+    """test(verbosity=1)
 
-
-    def endian(self):
-        """endian()
-
-return the bit endianness as a string (either 'little' or 'big')."""
-        return self.buffer_info()[2]
-
-
-def test():
+Selftest the module."""
     import test_bitarray
-    return test_bitarray.run()
+    return test_bitarray.run(verbosity=verbosity)
